@@ -37,12 +37,16 @@
 
 #include <iostream>
 
+#ifdef GLOM_ENABLE_CLIENT_ONLY
+#define NEW_PAGE 1
+#endif /* GLOM_ENABLE_CLIENT_ONLY */
+
 namespace
 {
 
-const char* RECENT_DUMMY_TEXT = _("No recently used documents available.");
-const char* TEMPLATE_DUMMY_TEXT = _("No templates available.");
-const char* NETWORK_DUMMY_TEXT = _("No sessions found on the local network.");
+const char* RECENT_DUMMY_TEXT = N_("No recently used documents available.");
+const char* TEMPLATE_DUMMY_TEXT = N_("No templates available.");
+const char* NETWORK_DUMMY_TEXT = N_("No sessions found on the local network.");
 
 /*bool has_dummy(const Gtk::TreeModel::iterator& parent, const std::auto_ptr<Gtk::TreeModel::iterator>& dummy)
 {
@@ -139,8 +143,10 @@ Dialog_ExistingOrNew::Dialog_ExistingOrNew(BaseObjectType* cobject, const Glib::
   m_iter_existing_other = m_existing_model->append();
   (*m_iter_existing_other)[m_existing_columns.m_col_title] = _("Select File");
 
+#ifndef G_OS_WIN32
   m_iter_existing_network = m_existing_model->append();
   (*m_iter_existing_network)[m_existing_columns.m_col_title] = _("Local Network");
+#endif
 
   m_iter_existing_recent = m_existing_model->append();
   (*m_iter_existing_recent)[m_existing_columns.m_col_title] = _("Recently Opened");
@@ -153,13 +159,19 @@ Dialog_ExistingOrNew::Dialog_ExistingOrNew(BaseObjectType* cobject, const Glib::
 
   // Load example files
 #ifdef G_OS_WIN32
-  gchar* dir = g_win32_get_package_installation_subdirectory(NULL, NULL, "share/doc/examples");
+  gchar* dir = g_win32_get_package_installation_subdirectory(NULL, NULL, "share/glom/doc/examples");
   std::string path(dir);
   g_free(dir);
+
+  if(!Glib::file_test(path, Glib::FILE_TEST_EXISTS))
+    path = GLOM_EXAMPLES_DIR;
 #else
+#ifndef GLOM_ENABLE_CLIENT_ONLY
   const char* path = GLOM_EXAMPLES_DIR;
 #endif
+#endif
 
+#ifndef GLOM_ENABLE_CLIENT_ONLY
   m_examples_dir = Gio::File::create_for_path(path);
 
   try
@@ -172,6 +184,7 @@ Dialog_ExistingOrNew::Dialog_ExistingOrNew(BaseObjectType* cobject, const Glib::
   {
     std::cerr << "Could not enumerate examples: " << ex.what() << std::endl;
   }
+#endif /* !GLOM_ENABLE_CLIENT_ONLY */
 
   // Browse local network
 #ifndef G_OS_WIN32
@@ -200,24 +213,36 @@ Dialog_ExistingOrNew::Dialog_ExistingOrNew(BaseObjectType* cobject, const Glib::
   if(children.begin() == children.end())
     m_iter_existing_recent_dummy = create_dummy_item_existing(m_iter_existing_recent, _(RECENT_DUMMY_TEXT));
 
+#ifndef G_OS_WIN32
   // Will be removed when items are added:
   m_iter_existing_network_dummy = create_dummy_item_existing(m_iter_existing_network, _(NETWORK_DUMMY_TEXT));
+#endif
+
   m_iter_new_template_dummy = create_dummy_item_new(m_iter_new_template, _(TEMPLATE_DUMMY_TEXT));
 
   // Expand recently used files
   m_existing_view->expand_row(m_existing_model->get_path(m_iter_existing_recent), false);
 
   m_select_button->signal_clicked().connect(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_select_clicked));
-  m_notebook->signal_switch_page().connect(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_switch_page));
+  m_select_button->set_image(*Gtk::manage(new Gtk::Image(Gtk::Stock::APPLY, Gtk::ICON_SIZE_BUTTON)));
 
+#ifndef GLOM_ENABLE_CLIENT_ONLY
+  m_notebook->signal_switch_page().connect(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_switch_page));
+#endif /* !GLOM_ENABLE_CLIENT_ONLY */
+    
   Glib::RefPtr<Gtk::TreeView::Selection> existing_view_selection = m_existing_view->get_selection();
   existing_view_selection->signal_changed().connect(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_existing_selection_changed));
   existing_view_selection->set_select_function( sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_existing_select_func) ); 
 
+#ifndef GLOM_ENABLE_CLIENT_ONLY
   Glib::RefPtr<Gtk::TreeView::Selection> new_view_selection = m_new_view->get_selection();
   new_view_selection->signal_changed().connect(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_new_selection_changed));
   new_view_selection->set_select_function( sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_new_select_func) );
-  
+#else /* GLOM_ENABLE_CLIENT_ONLY */
+  m_notebook->remove_page(NEW_PAGE);
+  m_notebook->set_show_tabs (false);
+#endif /* !GLOM_ENABLE_CLIENT_ONLY */
+    
   update_ui_sensitivity();
 }
 
@@ -255,12 +280,14 @@ Dialog_ExistingOrNew::~Dialog_ExistingOrNew()
 bool Dialog_ExistingOrNew::on_existing_select_func(const Glib::RefPtr<Gtk::TreeModel>& model, const Gtk::TreeModel::Path& path, bool path_currently_selected)
 {
   Gtk::TreeModel::iterator iter = model->get_iter(path);
+#ifndef G_OS_WIN32
   if(iter == m_iter_existing_network)
     return false; /* Do not allow parent nodes to be selected. */
-  else if(iter == m_iter_existing_recent)
+#endif
+  if(iter == m_iter_existing_recent)
     return false;
-  else
-    return true;
+
+  return true;
 }
 
 bool Dialog_ExistingOrNew::on_new_select_func(const Glib::RefPtr<Gtk::TreeModel>& model, const Gtk::TreeModel::Path& path, bool path_currently_selected)
@@ -283,12 +310,14 @@ Dialog_ExistingOrNew::Action Dialog_ExistingOrNew::get_action_impl(Gtk::TreeMode
     iter = m_existing_view->get_selection()->get_selected();
     if(m_existing_model->is_ancestor(m_iter_existing_recent, iter))
       return OPEN_URI;
-    else if(m_existing_model->is_ancestor(m_iter_existing_network, iter))
+#ifndef G_OS_WIN32
+    if(m_existing_model->is_ancestor(m_iter_existing_network, iter))
       return OPEN_REMOTE;
-    else if(iter == m_iter_existing_other)
+#endif
+    if(iter == m_iter_existing_other)
       return OPEN_URI;
-    else
-      return NONE;
+
+    return NONE;
   }
   else
   {
@@ -387,14 +416,18 @@ void Dialog_ExistingOrNew::existing_icon_data_func(Gtk::CellRenderer* renderer,
 
   if(iter == m_iter_existing_recent)
     pixbuf_renderer->property_stock_id() = Gtk::Stock::INDEX.id; // TODO: More meaningful icon?
+#ifndef G_OS_WIN32
   else if(iter == m_iter_existing_network)
     pixbuf_renderer->property_stock_id() = Gtk::Stock::NETWORK.id;
+#endif
   else if(iter == m_iter_existing_other)
     pixbuf_renderer->property_stock_id() = Gtk::Stock::OPEN.id;
   else if(m_iter_existing_recent_dummy.get() != NULL && iter == *m_iter_existing_recent_dummy)
     pixbuf_renderer->property_stock_id() = Gtk::Stock::DIALOG_ERROR.id; // TODO: Use Stock::STOP instead?
+#ifndef G_OS_WIN32
   else if(m_iter_existing_network_dummy.get() != NULL && iter == *m_iter_existing_network_dummy)
     pixbuf_renderer->property_stock_id() = Gtk::Stock::DIALOG_ERROR.id; // TODO: Use Stock::STOP instead?
+#endif
   else
   {
     if(m_existing_model->is_ancestor(m_iter_existing_recent, iter))
@@ -403,11 +436,13 @@ void Dialog_ExistingOrNew::existing_icon_data_func(Gtk::CellRenderer* renderer,
       //pixbuf_renderer->property_pixbuf() = (*info)->get_icon(Gtk::ICON_SIZE_BUTTON);
       pixbuf_renderer->set_property("icon-name", Glib::ustring("glom"));
     }
+#ifndef G_OS_WIN32
     else if(m_existing_model->is_ancestor(m_iter_existing_network, iter))
     {
       //pixbuf_renderer->property_stock_id() = Gtk::Stock::CONNECT.id;
       pixbuf_renderer->set_property("icon-name", Glib::ustring("glom"));
     }
+#endif
     else
     {
       throw std::logic_error("Unexpected iterator in existing_icon_data_func");
@@ -425,8 +460,12 @@ void Dialog_ExistingOrNew::existing_title_data_func(Gtk::CellRenderer* renderer,
   // Default: Use default color
   text_renderer->property_foreground_set() = false;
   // Use grey if parent item has no children
+#ifndef G_OS_WIN32
   if( (iter == m_iter_existing_network && m_iter_existing_network_dummy.get()) ||
       (iter == m_iter_existing_recent && m_iter_existing_recent_dummy.get()))
+#else
+  if(iter == m_iter_existing_recent && m_iter_existing_recent_dummy.get())
+#endif
   {
     text_renderer->property_foreground() = "grey";
   }
@@ -506,9 +545,15 @@ void Dialog_ExistingOrNew::update_ui_sensitivity()
     else
     {
       Gtk::TreeModel::iterator sel = m_existing_view->get_selection()->get_selected();
-      sensitivity = (sel != m_iter_existing_recent && sel != m_iter_existing_network &&
-                     (!m_iter_existing_recent_dummy.get() || sel != *m_iter_existing_recent_dummy) &&
-                     (!m_iter_existing_network_dummy.get() || sel != *m_iter_existing_network_dummy));
+      sensitivity = (sel != m_iter_existing_recent);
+#ifndef G_OS_WIN32
+      sensitivity = sensitivity && (sel != m_iter_existing_network);
+#endif
+
+      sensitivity = sensitivity && (!m_iter_existing_recent_dummy.get() || sel != *m_iter_existing_recent_dummy);
+#ifndef G_OS_WIN32
+      sensitivity = sensitivity && (!m_iter_existing_network_dummy.get() || sel != *m_iter_existing_network_dummy);
+#endif
     }
   }
   else
@@ -530,6 +575,7 @@ void Dialog_ExistingOrNew::update_ui_sensitivity()
   m_select_button->set_sensitive(sensitivity);
 }
 
+#ifndef GLOM_ENABLE_CLIENT_ONLY
 void Dialog_ExistingOrNew::on_enumerate_children(const Glib::RefPtr<Gio::AsyncResult>& res)
 {
   try
@@ -650,6 +696,7 @@ void Dialog_ExistingOrNew::on_stream_read(const Glib::RefPtr<Gio::AsyncResult>&
 
   m_examples_enumerator->next_files_async(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_next_files));
 }
+#endif /* !GLOM_ENABLE_CLIENT_ONLY */
 
 #ifndef G_OS_WIN32
 void Dialog_ExistingOrNew::on_service_found(const Glib::ustring& name, EpcServiceInfo* info)
@@ -695,13 +742,13 @@ void Dialog_ExistingOrNew::on_service_removed(const Glib::ustring& name, const G
 }
 #endif // !G_OS_WIN32
 
-void Dialog_ExistingOrNew::on_existing_row_activated(const Gtk::TreePath& path, Gtk::TreeViewColumn* column)
+void Dialog_ExistingOrNew::on_existing_row_activated(const Gtk::TreeModel::Path& path, Gtk::TreeViewColumn* column)
 {
   if(m_select_button->is_sensitive())
     on_select_clicked();
 }
 
-void Dialog_ExistingOrNew::on_existing_button_clicked(const Gtk::TreePath& path)
+void Dialog_ExistingOrNew::on_existing_button_clicked(const Gtk::TreeModel::Path& path)
 {
   m_existing_view->get_selection()->select(path);
 
@@ -709,13 +756,13 @@ void Dialog_ExistingOrNew::on_existing_button_clicked(const Gtk::TreePath& path)
     on_select_clicked();
 }
 
-void Dialog_ExistingOrNew::on_new_row_activated(const Gtk::TreePath& path, Gtk::TreeViewColumn* column)
+void Dialog_ExistingOrNew::on_new_row_activated(const Gtk::TreeModel::Path& path, Gtk::TreeViewColumn* column)
 {
   if(m_select_button->is_sensitive())
     on_select_clicked();
 }
 
-void Dialog_ExistingOrNew::on_new_button_clicked(const Gtk::TreePath& path)
+void Dialog_ExistingOrNew::on_new_button_clicked(const Gtk::TreeModel::Path& path)
 {
   m_new_view->get_selection()->select(path);
 
