@@ -21,7 +21,7 @@
 #include "db_adddel.h"
 #include <algorithm> //For std::find.
 #include <glibmm/i18n.h>
-#include "../cellrendererlist.h"
+#include "../cellrendererlist/cellrendererlist.h"
 #include "db_treeviewcolumn_glom.h"
 #include <glom/libglom/data_structure/glomconversions.h>
 #include "../../dialog_invalid_data.h"
@@ -71,9 +71,10 @@ DbAddDel::DbAddDel()
   m_bAllowUserActions(true),
   m_bPreventUserSignals(false),
   m_bIgnoreTreeViewSignals(false),
-  m_auto_add(true),
   m_allow_add(true),
   m_allow_delete(true),
+  m_find_mode(false),
+  m_allow_only_one_related_record(false),
   m_columns_ready(false),
   m_allow_view(true),
   m_allow_view_details(false),
@@ -95,7 +96,6 @@ DbAddDel::DbAddDel()
   m_ScrolledWindow.set_shadow_type(Gtk::SHADOW_IN);
 
   m_TreeView.set_fixed_height_mode(); //This allows some optimizations.
-  m_TreeView.set_headers_visible(true); //This is not the default in Maemo's GTK+. Bad Maemo.
   m_TreeView.show();
 
   pack_start(m_ScrolledWindow);
@@ -123,8 +123,13 @@ DbAddDel::DbAddDel()
 
   show_all_children();
 
+#ifdef GLOM_ENABLE_CLIENT_ONLY //Actually this has only been necessary for Maemo.
   // Adjust sizing when style changed
+  // TODO_Maemo: This calls construct_specified_columns(), which runs the SQL query again.
+  //       Try to change the row and column sizes without doing that.
+  
   signal_style_changed().connect(sigc::mem_fun(*this, &DbAddDel::on_self_style_changed));
+#endif // !GLOM_ENABLE_CLIENT_ONLY
 }
 
 DbAddDel::~DbAddDel()
@@ -187,18 +192,8 @@ void DbAddDel::on_MenuPopup_activate_layout()
 
 void DbAddDel::on_MenuPopup_activate_Add()
 {
-  if(m_auto_add)
-  {
-    Gtk::TreeModel::iterator iter = get_item_placeholder();
-    if(iter)
-    {
-      select_item(iter, true /* start_editing */);
-    }
-  }
-  else
-  {
-    signal_user_requested_add().emit(); //Let the client code add the row explicitly, if it wants.
-  }
+  //Create a new record in the database:
+  start_new_record();
 }
 
 void DbAddDel::on_MenuPopup_activate_Delete()
@@ -247,7 +242,7 @@ void DbAddDel::setup_menu()
   m_refActionGroup->add(m_refContextLayout,
     sigc::mem_fun(*this, &DbAddDel::on_MenuPopup_activate_layout) );
 
-  //TODO: This does not work until this widget is in a container in the window:s
+  //TODO: This does not work until this widget is in a container in the window:
   App_Glom* pApp = get_application();
   if(pApp)
   {
@@ -328,6 +323,7 @@ bool DbAddDel::on_button_press_event_Popup(GdkEventButton *event)
     pApp->add_developer_action(m_refContextLayout); //So that it can be disabled when not in developer mode.
     pApp->update_userlevel_ui(); //Update our action's sensitivity. 
   }
+#endif
 
   GdkModifierType mods;
   gdk_window_get_pointer( Gtk::Widget::gobj()->window, 0, 0, &mods );
@@ -349,7 +345,6 @@ bool DbAddDel::on_button_press_event_Popup(GdkEventButton *event)
       return false; //Not handled.
     }
   }
-#endif
 
   return  false; //Not handled. TODO: Call base class?
 }
@@ -363,7 +358,7 @@ Gtk::TreeModel::iterator DbAddDel::get_item_placeholder()
    return Gtk::TreeModel::iterator();
 }
 
-Gnome::Gda::Value DbAddDel::get_value(const Gtk::TreeModel::iterator& iter, const sharedptr<const LayoutItem_Field>& layout_item)
+Gnome::Gda::Value DbAddDel::get_value(const Gtk::TreeModel::iterator& iter, const sharedptr<const LayoutItem_Field>& layout_item) const
 {
   Gnome::Gda::Value value;
 
@@ -387,7 +382,7 @@ Gnome::Gda::Value DbAddDel::get_value(const Gtk::TreeModel::iterator& iter, cons
   return value;
 }
 
-Gnome::Gda::Value DbAddDel::get_value_key_selected()
+Gnome::Gda::Value DbAddDel::get_value_key_selected() const
 {
   Gtk::TreeModel::iterator iter = get_item_selected();
   if(iter)
@@ -398,7 +393,7 @@ Gnome::Gda::Value DbAddDel::get_value_key_selected()
     return Gnome::Gda::Value();
 }
 
-Gnome::Gda::Value DbAddDel::get_value_selected(const sharedptr<const LayoutItem_Field>& layout_item)
+Gnome::Gda::Value DbAddDel::get_value_selected(const sharedptr<const LayoutItem_Field>& layout_item) const
 {
   return get_value(get_item_selected(), layout_item);
 }
@@ -417,6 +412,21 @@ Gtk::TreeModel::iterator DbAddDel::get_item_selected()
     return Gtk::TreeModel::iterator();
 }
 
+Gtk::TreeModel::iterator DbAddDel::get_item_selected() const
+{
+  Glib::RefPtr<const Gtk::TreeSelection> refTreeSelection = m_TreeView.get_selection();
+  if(refTreeSelection)
+  {
+     Glib::RefPtr<Gtk::TreeSelection> unconst = Glib::RefPtr<Gtk::TreeSelection>::cast_const(refTreeSelection);
+     return unconst->get_selected();
+  }
+
+  if(m_refListStore)
+    return m_refListStore->children().end();
+  else
+    return Gtk::TreeModel::iterator();
+}
+
 
 Gtk::TreeModel::iterator DbAddDel::get_row(const Gnome::Gda::Value& key)
 {
@@ -623,6 +633,11 @@ Gtk::CellRenderer* DbAddDel::construct_specified_columns_cellrenderer(const shar
         break;
       }
     } //switch
+
+    //Set font and colors:
+    const FieldFormatting& formatting = item_field->get_formatting_used();
+    if(pCellRenderer)
+      apply_formatting(pCellRenderer, formatting);
   }
   else
   {
@@ -633,7 +648,7 @@ Gtk::CellRenderer* DbAddDel::construct_specified_columns_cellrenderer(const shar
      {
        Gtk::CellRendererPixbuf* pixbuf_renderer = Gtk::manage( new Gtk::CellRendererPixbuf() );
 
-       Glib::RefPtr<Gdk::Pixbuf> pixbuf = Conversions::get_pixbuf_for_gda_value(item_image->get_image());
+       Glib::RefPtr<Gdk::Pixbuf> pixbuf = item_image->get_image_as_pixbuf();
        if(pixbuf)
          pixbuf_renderer->set_property("pixbuf", pixbuf);
        else
@@ -785,6 +800,26 @@ Gtk::CellRenderer* DbAddDel::construct_specified_columns_cellrenderer(const shar
   return pCellRenderer;
 }
 
+void DbAddDel::apply_formatting(Gtk::CellRenderer* renderer, const FieldFormatting& formatting)
+{
+  Gtk::CellRendererText* text_renderer = dynamic_cast<Gtk::CellRendererText*>(renderer);
+  if(!text_renderer)
+    return;
+
+  //Use the text formatting:
+  const Glib::ustring font_desc = formatting.get_text_format_font();
+  if(!font_desc.empty())
+    text_renderer->property_font() = font_desc;
+
+  const Glib::ustring fg = formatting.get_text_format_color_foreground();
+  if(!fg.empty())
+    text_renderer->property_foreground() = fg;
+
+  const Glib::ustring bg = formatting.get_text_format_color_background();
+  if(!bg.empty())
+    text_renderer->property_background() = bg;
+}
+
 void DbAddDel::construct_specified_columns()
 {
   //std::cout << "debug: DbAddDel::construct_specified_columns" << std::endl;
@@ -808,7 +843,7 @@ void DbAddDel::construct_specified_columns()
   //Create the Gtk ColumnRecord:
 
   Gtk::TreeModel::ColumnRecord record;
-
+    
   //Database columns:
   type_model_store::type_vec_fields fields;
   {
@@ -834,6 +869,8 @@ void DbAddDel::construct_specified_columns()
       }
     }
   }
+  
+  m_FieldsShown = fields; //Needed by Base_DB_Table_Data::record_new().
 
   //Find the primary key:
   int column_index_key = 0;
@@ -870,6 +907,7 @@ void DbAddDel::construct_specified_columns()
     m_refListStore = Glib::RefPtr<type_model_store>();
   }
 
+ 
   m_TreeView.set_model(m_refListStore);
 
 
@@ -1024,7 +1062,7 @@ void DbAddDel::set_value(const Gtk::TreeModel::iterator& iter, const sharedptr<c
       type_list_indexes list_indexes = get_data_model_column_index(layout_item);
       for(type_list_indexes::const_iterator iter = list_indexes.begin(); iter != list_indexes.end(); ++iter)
       {
-        guint treemodel_col = *iter + get_count_hidden_system_columns();
+        const guint treemodel_col = *iter + get_count_hidden_system_columns();
         treerow.set_value(treemodel_col, value);
 
         //Mark this row as not a placeholder because it has real data now.
@@ -1058,6 +1096,7 @@ void DbAddDel::remove_all_columns()
 void DbAddDel::set_table_name(const Glib::ustring& table_name)
 {
   m_found_set.m_table_name = table_name;
+  Base_DB_Table::m_table_name = table_name;
 }
 
 guint DbAddDel::add_column(const sharedptr<LayoutItem>& layout_item)
@@ -1243,6 +1282,16 @@ void DbAddDel::set_show_column_titles(bool bVal)
   m_TreeView.set_headers_visible(bVal);
 }
 
+void DbAddDel::set_find_mode(bool val)
+{
+  m_find_mode = val;
+}
+  
+void DbAddDel::set_allow_only_one_related_record(bool val)
+{
+  m_allow_only_one_related_record = val;
+}
+
 
 void DbAddDel::set_column_width(guint /* col */, guint /*width*/)
 {
@@ -1326,7 +1375,7 @@ DbAddDel::InnerIgnore::~InnerIgnore()
   m_pOuter = false;
 }
 
-Gnome::Gda::Value DbAddDel::treeview_get_key(const Gtk::TreeModel::iterator& row)
+Gnome::Gda::Value DbAddDel::treeview_get_key(const Gtk::TreeModel::iterator& row) const
 {
   Gnome::Gda::Value value;
 
@@ -1348,7 +1397,7 @@ void DbAddDel::on_treeview_cell_edited_bool(const Glib::ustring& path_string, in
   if(!m_refListStore)
     return;
 
-  Gtk::TreePath path(path_string);
+  const Gtk::TreeModel::Path path(path_string);
 
   //Get the row from the path:
   Gtk::TreeModel::iterator iter = m_refListStore->get_iter(path);
@@ -1356,13 +1405,13 @@ void DbAddDel::on_treeview_cell_edited_bool(const Glib::ustring& path_string, in
   {
     Gtk::TreeModel::Row row = *iter;
 
-    int tree_model_column_index = data_model_column_index + get_count_hidden_system_columns();
+    const int tree_model_column_index = data_model_column_index + get_count_hidden_system_columns();
 
     Gnome::Gda::Value value_old;
     row.get_value(tree_model_column_index, value_old);
 
     const bool bValueOld = (value_old.get_value_type() == G_TYPE_BOOLEAN) && value_old.get_boolean();
-    bool bValueNew = !bValueOld;
+    const bool bValueNew = !bValueOld;
     Gnome::Gda::Value value_new;
     value_new.set(bValueNew);
     //Store the user's new value in the model:
@@ -1411,13 +1460,13 @@ void DbAddDel::on_treeview_cell_edited_bool(const Glib::ustring& path_string, in
          
       //Signal that a new key was added:
       //We will ignore editing of bool values in the blank row. It seems like a bad way to start a new record.
-      //m_signal_user_added.emit(row);
+      //user_added(row);
     }
     else if(bIsChange)
     {
       //Existing item changed:
 
-      m_signal_user_changed.emit(row, model_column_index);
+      user_changed(row, model_column_index);
     }
   }
 }
@@ -1431,7 +1480,7 @@ void DbAddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const G
   if(!m_refListStore)
     return;
 
-  Gtk::TreePath path(path_string);
+  const Gtk::TreeModel::Path path(path_string);
 
   //Get the row from the path:
   Gtk::TreeModel::iterator iter = m_refListStore->get_iter(path);
@@ -1443,6 +1492,7 @@ void DbAddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const G
 
     Gnome::Gda::Value valOld;
     row.get_value(treemodel_column_index, valOld);
+    //std::cout << "debug: valOld type=" << valOld.get_value_type() << std::endl;
 
     //Store the user's new text in the model:
     //row.set_value(treemodel_column_index, new_text);
@@ -1450,7 +1500,7 @@ void DbAddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const G
     //Is it an add or a change?:
     bool bIsAdd = false;
     bool bIsChange = false;
-    bool do_signal = true;
+    bool do_change = true;
 
     if(get_allow_user_actions()) //If add is possible:
     {
@@ -1480,7 +1530,7 @@ void DbAddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const G
       //Make sure that the entered data is suitable for this field type:
       bool success = false;
 
-      Gnome::Gda::Value value = Conversions::parse_value(field_type, new_text, item_field->get_formatting_used().m_numeric_format, success);
+      const Gnome::Gda::Value value = Conversions::parse_value(field_type, new_text, item_field->get_formatting_used().m_numeric_format, success);
       if(!success)
       {
           //Tell the user and offer to revert or try again:
@@ -1533,12 +1583,14 @@ void DbAddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const G
             }
           }
 
-          do_signal = false;
+          do_change = false;
       }
       else
       {
         //Store the value in the model:
+        //std::cout << "debug: setting value: column=" << treemodel_column_index << ", type=" << value.get_value_type() << std::endl;
         row.set_value(treemodel_column_index, value);
+        //std::cout << "debug: after setting value" << std::endl;
       }
 
       if(!bIsAdd)
@@ -1547,9 +1599,9 @@ void DbAddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const G
       //Fire appropriate signal:
       if(bIsAdd)
       {
-        //Signal that a new key was added"
+        //Signal that a new key was added:
         if(m_allow_add)
-          m_signal_user_added.emit(row, model_column_index);
+          user_added(row);
       }
       else if(bIsChange)
       {
@@ -1557,24 +1609,14 @@ void DbAddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const G
         //Check that it has really changed - get the last value.
         if(value != valOld)
         {
-          if(do_signal)
-            m_signal_user_changed.emit(row, model_column_index);
+          if(do_change)
+            user_changed(row, model_column_index);
         }
       }
     }
   }
 }
 
-DbAddDel::type_signal_user_added DbAddDel::signal_user_added()
-{
-  return m_signal_user_added;
-}
-
-DbAddDel::type_signal_user_changed DbAddDel::signal_user_changed()
-{
-  return m_signal_user_changed;
-}
-
 #ifndef GLOM_ENABLE_CLIENT_ONLY
 DbAddDel::type_signal_user_requested_layout DbAddDel::signal_user_requested_layout()
 {
@@ -1592,70 +1634,19 @@ DbAddDel::type_signal_user_requested_edit DbAddDel::signal_user_requested_edit()
   return m_signal_user_requested_edit;
 }
 
-DbAddDel::type_signal_user_requested_add DbAddDel::signal_user_requested_add()
-{
-  return m_signal_user_requested_add;
-}
-
-DbAddDel::type_signal_user_activated DbAddDel::signal_user_activated()
-{
-  return m_signal_user_activated;
-}
-
-DbAddDel::type_signal_user_reordered_columns DbAddDel::signal_user_reordered_columns()
-{
-  return m_signal_user_reordered_columns;
-}
 
 DbAddDel::type_signal_script_button_clicked DbAddDel::signal_script_button_clicked()
 {
   return m_signal_script_button_clicked;
 }
 
+DbAddDel::type_signal_record_added DbAddDel::signal_record_added()
+{
+  return m_signal_record_added;
+}
+
 void DbAddDel::on_treeview_button_press_event(GdkEventButton* event)
 {
-  if(event->type == GDK_BUTTON_PRESS) //Whatever would cause cellrenderer activation.
-  {
-    //This is really horrible code:
-    //Maybe we can improve the gtkmm API for this:
-
-    //Get the row and column:
-    Gtk::TreeModel::Path path;
-    Gtk::TreeView::Column* pColumn = 0;
-    int cell_x = 0;
-    int cell_y = 0;  
-
-    // Make sure to use the non-deprecated const version:
-    bool row_exists = static_cast<const Gtk::TreeView&>(m_TreeView).get_path_at_pos((int)event->x, (int)event->y, path, pColumn, cell_x, cell_y);
-
-    //Get the row:
-    if(row_exists)
-    {
-      if(!m_refListStore)
-        return;
-
-      Gtk::TreeModel::iterator iterRow = m_refListStore->get_iter(path);
-      if(iterRow)
-      {
-        //Get the column:
-        int tree_col = 0;
-        int col_index = get_count_hidden_system_columns();
-
-        typedef std::vector<Gtk::TreeView::Column*> type_vecTreeViewColumns;
-        type_vecTreeViewColumns vecColumns = m_TreeView.get_columns();
-        for(type_vecTreeViewColumns::const_iterator iter = vecColumns.begin(); iter != vecColumns.end(); iter++)
-        {
-          if(*iter == pColumn)
-            tree_col = col_index; //Found.
-
-          col_index++;
-        }
-
-        signal_user_activated().emit(iterRow, tree_col);
-      }
-    }
-  }
-
   on_button_press_event_Popup(event);
 }
 
@@ -1700,7 +1691,7 @@ guint DbAddDel::treeview_append_column(const Glib::ustring& title, Gtk::CellRend
   pViewColumn->set_resizable();
   
   guint column_width = 0;
-  if(!layout_item->get_display_width(column_width)) //Not saved in document, but remembered when the column is resized.
+  if(!layout_item->get_display_width(column_width))
      column_width = 100; //Fairly sensible default. TODO: Choose a width based on the first 100 values.
 
   pViewColumn->set_fixed_width((int)column_width); //This is the only way to set the width, so we need to set it as resizable again immediately afterwards.
@@ -1729,9 +1720,17 @@ void DbAddDel::on_treeview_column_resized(int model_column_index, DbTreeViewColu
   if(!view_column)
     return;
 
+  //We do not save the column width if this is the last column, 
+  //because that must always be automatic, 
+  //because it must resize when the whole column resizes.
+  std::list<const Gtk::TreeView::Column*> columns = m_TreeView.get_columns();
+  const int n_view_columns = columns.size();
+  if(n_view_columns && (view_column == m_TreeView.get_column(n_view_columns -1)))
+    return;
+
   DbAddDelColumnInfo& column_info = m_ColumnTypes[model_column_index];
 
-  guint width = (guint)view_column->get_width();
+  const guint width = (guint)view_column->get_width();
   //std::cout << "  DbAddDel::on_treeview_column_resized(): width=" << width << std::endl;
 
   if(column_info.m_item)
@@ -1740,6 +1739,8 @@ void DbAddDel::on_treeview_column_resized(int model_column_index, DbTreeViewColu
 
 void DbAddDel::on_treeview_column_clicked(int model_column_index)
 {
+  Bakery::BusyCursor busy_cursor(get_application());
+
   if(model_column_index >= (int)m_ColumnTypes.size())
     return;
 
@@ -1788,7 +1789,7 @@ void DbAddDel::on_treeview_columns_changed()
     }
 
     //Tell other code that something has changed, so the new column order can be serialized.
-    m_signal_user_reordered_columns.emit();
+    //TODO: If this is ever wanted: m_signal_user_reordered_columns.emit();
   }
 }
 
@@ -1798,11 +1799,6 @@ DbAddDel::type_vecStrings DbAddDel::get_columns_order() const
   return m_vecColumnIDs;
 }
 
-void DbAddDel::set_auto_add(bool value)
-{
-  m_auto_add = value;
-}
-
 Glib::RefPtr<Gtk::TreeModel> DbAddDel::get_model()
 {
   return m_refListStore;
@@ -1847,7 +1843,7 @@ Gtk::TreeModel::iterator DbAddDel::get_last_row()
     return Gtk::TreeModel::iterator();
 }
 
-Gnome::Gda::Value DbAddDel::get_value_key(const Gtk::TreeModel::iterator& iter)
+Gnome::Gda::Value DbAddDel::get_value_key(const Gtk::TreeModel::iterator& iter) const
 {
   return treeview_get_key(iter);
 }
@@ -1903,7 +1899,7 @@ bool DbAddDel::get_model_column_index(guint view_column_index, guint& model_colu
   return true;
 }
 
-bool DbAddDel::get_view_column_index(guint model_column_index, guint& view_column_index)
+bool DbAddDel::get_view_column_index(guint model_column_index, guint& view_column_index) const
 {
   //Initialize output parameter:
   view_column_index = 0;
@@ -1925,7 +1921,7 @@ bool DbAddDel::get_view_column_index(guint model_column_index, guint& view_colum
   return true;
 }
 
-guint DbAddDel::get_count_hidden_system_columns()
+guint DbAddDel::get_count_hidden_system_columns() const
 {
   return 0; //The key now has explicit API in the model.
   //return 1; //The key.
@@ -1949,6 +1945,9 @@ void DbAddDel::set_key_field(const sharedptr<Field>& field)
 
 void DbAddDel::treeviewcolumn_on_cell_data(Gtk::CellRenderer* renderer, const Gtk::TreeModel::iterator& iter, int model_column_index, int data_model_column_index)
 {
+  //std::cout << "debug: DbAddDel::treeviewcolumn_on_cell_data()" << std::endl; 
+
+
   if(iter)
   {
     const DbAddDelColumnInfo& column_info = m_ColumnTypes[model_column_index];
@@ -1961,6 +1960,13 @@ void DbAddDel::treeviewcolumn_on_cell_data(Gtk::CellRenderer* renderer, const Gt
       Gnome::Gda::Value value;
       treerow->get_value(col_real, value);
 
+      /*
+      GType debug_type = value.get_value_type();
+      std::cout << "  debug: DbAddDel::treeviewcolumn_on_cell_data(): GdaValue from TreeModel::get_value(): GType=" << debug_type << std::endl;
+      if(debug_type)
+         std::cout << "    GType name=\"" << g_type_name(debug_type) << "\"" << std::endl; 
+      */
+
       switch(field->get_glom_type())
       {
         case(Field::TYPE_BOOLEAN):
@@ -1995,6 +2001,7 @@ void DbAddDel::treeviewcolumn_on_cell_data(Gtk::CellRenderer* renderer, const Gt
           Gtk::CellRendererText* pDerived = dynamic_cast<Gtk::CellRendererText*>(renderer);
           if(pDerived)
           {
+            //std::cout << "  debug: DbAddDel::treeviewcolumn_on_cell_data(): field name=" << field->get_name() << ", glom type=" << field->get_glom_type() << std::endl;
             const Glib::ustring text = Conversions::get_text_for_gda_value(field->get_glom_type(), value, field->get_formatting_used().m_numeric_format);
             //g_assert(text != "NULL");
             g_object_set(pDerived->gobj(), "text", text.c_str(), (gpointer)NULL);
@@ -2048,13 +2055,17 @@ void DbAddDel::on_cell_button_clicked(const Gtk::TreeModel::Path& path)
   on_MenuPopup_activate_Edit();
 }
 
+#ifdef GLOM_ENABLE_CLIENT_ONLY 
 void DbAddDel::on_self_style_changed(const Glib::RefPtr<Gtk::Style>& style)
 {
-  // Reset fixed cell height because the font might have changed due to the new style
+  // Reset fixed cell height because the font might have changed due to the new style:
   m_fixed_cell_height = 0;
-  // Reconstruct columns because sizes might have changed
+
+  // Reconstruct columns because sizes might have changed:
+  // (TODO: But don't get the data again because that would be inefficient).
   construct_specified_columns();
 }
+#endif
 
 void DbAddDel::set_open_button_title(const Glib::ustring& title)
 {
@@ -2076,6 +2087,337 @@ void DbAddDel::show_hint_model()
   m_TreeView.append_column("", m_columns_hint.m_col_hint);
 }
 
+bool DbAddDel::start_new_record()
+{
+  Gtk::TreeModel::iterator iter = get_item_placeholder();
+  if(!iter)
+    return false;
+  
+  sharedptr<LayoutItem_Field> fieldToEdit;
+
+  //Start editing in the primary key or the first cell if the primary key is auto-incremented (because there is no point in editing an auto-generated value)
+  //guint index_primary_key = 0;
+  const bool bPresent = true; //get_field_primary_key_index(index_primary_key); //If there is no primary key then the default of 0 is OK.
+  if(!bPresent)
+    return false;
+   
+  sharedptr<Field> fieldPrimaryKey = get_key_field();
+  if(fieldPrimaryKey && fieldPrimaryKey->get_auto_increment())
+  {
+    //Start editing in the first cell that is not auto_increment:
+    for(type_ColumnTypes::iterator iter = m_ColumnTypes.begin(); iter != m_ColumnTypes.end(); ++iter)
+    {
+      sharedptr<LayoutItem> layout_item = iter->m_item;
+      sharedptr<LayoutItem_Field> layout_item_field = sharedptr<LayoutItem_Field>::cast_dynamic(layout_item);
+      if(!(layout_item_field->get_full_field_details()->get_auto_increment()))
+      {
+        fieldToEdit = layout_item_field;
+        break;
+      }
+    }
+  }
+  else
+  {
+    //The primary key is not auto-increment, so start by editing it:
+    fieldToEdit = sharedptr<LayoutItem_Field>::create();
+    fieldToEdit->set_full_field_details(fieldPrimaryKey);
+  }
+
+  //std::cout << "debug: index_field_to_edit=" << index_field_to_edit << std::endl;
+
+  if(fieldToEdit)
+  {
+    select_item(iter, fieldToEdit, true /* start_editing */);
+  }
+  else
+  {
+    std::cout << "start_new_record(): no editable rows." << std::endl;
+    //The only keys are non-editable, so just add a row:
+    select_item(iter); //without start_editing.
+    //g_warning("start_new_record(): index_field_to_edit does not exist: %d", index_field_to_edit);
+  }
+  
+  return true;
+}
+
+void DbAddDel::user_changed(const Gtk::TreeModel::iterator& row, guint col)
+{
+  const Gnome::Gda::Value parent_primary_key_value = get_value_key(row);
+  sharedptr<const LayoutItem_Field> layout_field = get_column_field(col);
+
+  if(!Conversions::value_is_empty(parent_primary_key_value)) //If the record's primary key is filled in:
+  {
+    //Just update the record:
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+    try
+#endif // GLIBMM_EXCEPTIONS_ENABLED
+    {
+      
+      Glib::ustring table_name = m_found_set.m_table_name;
+      sharedptr<Field> primary_key_field;
+      Gnome::Gda::Value primary_key_value;
+
+      if(!layout_field->get_has_relationship_name())
+      {
+        table_name = m_found_set.m_table_name;
+        primary_key_field = get_key_field();
+        primary_key_value = parent_primary_key_value;
+      }
+      else
+      {
+        //If it's a related field then discover the actual table that it's in,
+        //plus how to identify the record in that table.
+        const Glib::ustring relationship_name = layout_field->get_relationship_name();
+
+        Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
+
+        sharedptr<Relationship> relationship = document->get_relationship(m_found_set.m_table_name, relationship_name);
+        if(relationship)
+        {
+          table_name = relationship->get_to_table();
+          const Glib::ustring to_field_name = relationship->get_to_field();
+          //Get the key field in the other table (the table that we will change)
+          primary_key_field = get_fields_for_table_one_field(table_name, to_field_name); //TODO_Performance.
+          if(primary_key_field)
+          {
+            //Get the value of the corresponding key in the current table (that identifies the record in the table that we will change)
+            sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
+            layout_item->set_full_field_details( document->get_field(relationship->get_from_table(), relationship->get_from_field()) );
+
+            primary_key_value = get_value_selected(layout_item);
+
+            //Note: This just uses an existing record if one already exists:
+            Gnome::Gda::Value primary_key_value_used;
+            const bool test = add_related_record_for_field(layout_field, relationship, primary_key_field, primary_key_value, primary_key_value_used);
+            if(!test)
+              return;
+
+            //Get the new primary_key_value if it has been created:
+            primary_key_value = primary_key_value_used;
+
+            //Now that the related record exists, the following code to set the value of the other field in the related field can succeed.
+          }
+          else
+          {
+            g_warning("Box_Data_List::on_flowtable_field_edited(): key not found for edited related field.");
+          }
+        }
+      }
+
+      //Update the field in the record (the record with this primary key):
+      const Gnome::Gda::Value field_value = get_value(row, layout_field);
+      //std::cout << "Box_Data_List::on_adddel_user_changed(): field_value = " << field_value.to_string() << std::endl;
+      //const sharedptr<const Field>& field = layout_field->m_field;
+      //const Glib::ustring strFieldName = layout_field->get_name();
+
+      LayoutFieldInRecord field_in_record(layout_field, m_found_set.m_table_name /* parent */, primary_key_field, primary_key_value);
+
+      //Check whether the value meets uniqueness constraints:
+      Gtk::Window* window = get_application();
+      if(!check_entered_value_for_uniqueness(m_found_set.m_table_name, row, layout_field, field_value, window))
+      {
+        //Revert to the value in the database:
+        const Gnome::Gda::Value value_old = get_field_value_in_database(field_in_record, window);
+        set_entered_field_data(row, layout_field, value_old);
+
+        return; //The value has been reverted to the value in the database.
+      }
+
+
+      const bool bTest = set_field_value_in_database(field_in_record, row, field_value, false /* don't use current calculations */, window);
+
+      //Glib::ustring strQuery = "UPDATE \"" + table_name + "\"";
+      //strQuery += " SET " +  /* table_name + "." + postgres does not seem to like the table name here */ strFieldName + " = " + field.sql(field_value);
+      //strQuery += " WHERE " + table_name + "." + primary_key_field.get_name() + " = " + primary_key_field.sql(primary_key_value);
+      //bool bTest = query_execute(strQuery);
+      if(!bTest)
+      {
+        //Update failed.
+        fill_from_database(); //Replace with correct values.
+      }
+      else
+        signal_record_changed().emit();
+    }
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+    catch(const Glib::Exception& ex)
+    {
+      handle_error(ex);
+    }
+    catch(const std::exception& ex)
+    {
+      handle_error(ex);
+    }
+#endif // GLIBMM_EXCEPTIONS_ENABLED
+  }
+  else
+  {
+    //This record probably doesn't exist yet.
+    //Add new record, which will generate the primary key:
+    user_added(row);
+    
+    const Gnome::Gda::Value primaryKeyValue = get_value_key(row); //TODO_Value
+    if(!(Conversions::value_is_empty(primaryKeyValue))) //If the Add succeeeded:
+    {
+      if(!(layout_field->get_full_field_details()->get_primary_key())) //Don't try to re-set the primary key field, because we just inserted the record with it.
+      {
+        user_changed(row, col); //Change this field in the new record.
+      }
+    }
+    else
+    {
+      //A field value was entered, but the record has not been added yet, because not enough information exists yet.
+       g_warning("Box_Data_List::on_adddel_user_changed(): debug: record not yet added.");
+    }
+  }
+}
+
+
+void DbAddDel::user_added(const Gtk::TreeModel::iterator& row)
+{
+  //Prevent impossible multiple related records:
+  if(m_allow_only_one_related_record && (get_count() > 0))
+  {
+    //Tell user that they can't do that:
+    Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Extra Related Records Not Possible")), true, Gtk::MESSAGE_WARNING);
+    dialog.set_secondary_text(_("You attempted to add a new related record, but there can only be one related record, because the relationship uses a unique key.")),
+    dialog.set_transient_for(*App_Glom::get_application());
+    dialog.run();
+    
+    return;
+  }
+  
+  //std::cout << "DbAddDel::on_adddel_user_added" << std::endl;
+
+  Gnome::Gda::Value primary_key_value;
+
+  sharedptr<Field> primary_key_field = get_key_field();
+
+  //Get the new primary key value, if one is available now:
+  if(primary_key_field->get_auto_increment())
+  {
+    //Auto-increment is awkward (we can't get the last-generated ID) with postgres, so we auto-generate it ourselves;
+    const Glib::ustring& strPrimaryKeyName = primary_key_field->get_name();
+    primary_key_value = get_next_auto_increment_value(m_found_set.m_table_name, strPrimaryKeyName);  //TODO: return a Gnome::Gda::Value of an appropriate type.
+  }
+  else
+  {
+    //Use the user-entered primary key value:
+
+    //This only works when the primary key is already stored: primary_key_value = get_value_key(row);
+    //sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
+    //layout_item->set_full_field_details(field);
+    
+    primary_key_value = get_value_key_selected();
+  }
+
+  //If no primary key value is available yet, then don't add the record yet:
+  if(!Conversions::value_is_empty(primary_key_value))
+  {
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+    sharedptr<SharedConnection> sharedconnection = connect_to_server(get_application()); //Keep it alive while we need the data_model.
+#else
+    std::auto_ptr<ExceptionConnection> error;
+    sharedptr<SharedConnection> sharedconnection = connect_to_server(get_app_window(), error); //Keep it alive while we need the data_model.
+    // Ignore error, sharedconnection presence is checked below
+#endif
+    if(sharedconnection)
+    {
+      sharedptr<LayoutItem_Field> layout_field = sharedptr<LayoutItem_Field>::create();
+      layout_field->set_full_field_details(primary_key_field);
+      if(!check_entered_value_for_uniqueness(m_found_set.m_table_name, layout_field, primary_key_value, get_application()))
+      {
+        //Revert to a blank value.
+        primary_key_value = Conversions::get_empty_value(layout_field->get_full_field_details()->get_glom_type());
+        set_entered_field_data(row, layout_field, primary_key_value);
+        return;
+      }
+
+      Glib::RefPtr<Gnome::Gda::DataModel> data_model = record_new(true /* use entered field data*/, primary_key_value);
+      if(data_model)
+      {
+        //Save the primary key value for later use:
+        //record_new() did this: set_value_key(row, primary_key_value);
+
+        //Show the primary key in the row, if the primary key is visible:
+
+        //If it's an auto-increment, then get the value and show it:
+        if(primary_key_field->get_auto_increment())
+        {
+          sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
+          layout_item->set_full_field_details(primary_key_field);
+          set_value(row, layout_item, primary_key_value);
+        }
+
+        //Allow a parent widget to link the new record by setting the foreign key:
+        signal_record_added().emit(row, primary_key_value);
+      }
+      else
+        handle_error();
+    }
+    else
+    {
+      //Add Record failed.
+      //Replace with correct values:
+      fill_from_database();
+    }
+  }
+}
+
+void DbAddDel::user_requested_delete(const Gtk::TreeModel::iterator& rowStart, const Gtk::TreeModel::iterator&  /* rowEnd TODO */)
+{
+  if(rowStart)
+  {
+    if(confirm_delete_record())
+    {
+      const Gnome::Gda::Value primary_key_value = get_primary_key_value(rowStart);
+      record_delete(primary_key_value);
+
+      //Remove the row:
+      remove_item(rowStart);
+
+      //TODO_refactor: Just emit signal_record_changed() directly instead?
+      on_record_deleted(primary_key_value);
+    }
+  }
+}
+
+//An override of the Base_DB method:
+void DbAddDel::set_entered_field_data(const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& value)
+{
+  return set_value_selected(field, value);
+}
+
+//An override of the Base_DB method:
+void DbAddDel::set_entered_field_data(const Gtk::TreeModel::iterator& row, const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& value)
+{
+  return set_value(row, field, value);
+}
+
+Gnome::Gda::Value DbAddDel::get_entered_field_data(const sharedptr<const LayoutItem_Field>& field) const
+{
+  return get_value_selected(field);
+}
+
+sharedptr<Field> DbAddDel::get_field_primary_key() const
+{
+  return get_key_field();
+}
+
+Gnome::Gda::Value DbAddDel::get_primary_key_value_selected() const
+{
+  return get_value_key_selected();
+}
+
+void DbAddDel::set_primary_key_value(const Gtk::TreeModel::iterator& row, const Gnome::Gda::Value& value)
+{
+  set_value_key(row, value);
+}
+
+Gnome::Gda::Value DbAddDel::get_primary_key_value(const Gtk::TreeModel::iterator& row) const
+{
+  return get_value_key(row);
+}
+
 } //namespace Glom
 
 
